\subsection{Composers example}

> {-# LANGUAGE MultiParamTypeClasses, ScopedTypeVariables #-}

> module Composers where
> import Data.List as List
> import Data.Set as Set
> import Control.Monad.State as State
> import Control.Monad.Identity
> import SLens
> import StateTBX

Here is a version of the familiar Composers example
\cite{composers},
see the Bx wiki;
versions of this have been used in many papers including e.g. the
Symmetric Lens paper \cite{symlens}.

Assumption: Name is a key in all our datastructures: the user is
required not to give as argument any view that contains more than one
element for a given name.

NB We are not saying this version is better than any other version:
it's just an illustration.

> composers :: SLens  [(Name,Dates)]
>                     (Set (Name, Nation, Dates)) 
>                     [(Name,Nation)]
> composers = SLens {  putr = putMN , putl = putNM, 
>                      missing = [] }
>   where
>     putMN (m, c) = (n, c')
>       where
>         n = selectNNfromNND tripleList
>         c' = selectNDfromNND tripleList
>         tripleList = h c [] (Set.toList m)
>         h [] sel leftover = reverse sel ++ sort leftover
>         h ((nn,_):cs) ss ls = h cs (ps ++ ss) ns
>            where (ps, ns) = selectNNDonKey nn ls 
>     putNM (n, c) = (m, c')
>       where
>       m = Set.fromList tripleList
>       c' = selectNDfromNND tripleList
>       tripleList = k n [] c 
>       k [] selected _ = List.reverse selected 
>       k (h@(nn,_):nts) ss ls = k nts (newTriple : ss) ns
>         where (ps, ns) = selectNDonKey nn ls
>               newTriple = newTripleFromList h (\(_,d) -> d) ps

where the useful `select' statements are packaged as follows

> type NND = (Name, Nation, Dates)
> selectNNDonKey :: Name -> [NND] -> ([NND],[NND]) 
> selectNNDonKey n = List.partition (\(nn,_,_) -> nn == n)
>
> type ND = (Name, Dates)
> selectNDonKey :: Name -> [ND] -> ([ND],[ND]) 
> selectNDonKey n = List.partition (\(nn,_) -> nn == n)
>
> selectNDfromNND :: [NND] -> [ND] 
> selectNDfromNND  = List.map (\(nn,nt,dd) -> (nn,dd)) 
>
> type NN = (Name, Nation)
> selectNNfromNND :: [NND] -> [NN] 
> selectNNfromNND  = List.map (\(nn,nt,dd) -> (nn,nt)) 
> mkNNDfromNN :: [NN] -> [NND] 
> mkNNDfromNN = List.map (\(nn,nt) -> (nn,nt,dates0))

This last helper function abstracts how to make a new triple

> newTripleFromList :: NN -> (a -> Dates) -> [a] -> NND 
> newTripleFromList (nn,nt) _ [] = (nn,nt,dates0)
> newTripleFromList (nn,nt) f (a:_) = (nn,nt,f a)

Now here is the same functionality as a bx. There are no effects other
than the state ones induced directly by the BX, so the underlying
monad is the Identity monad.

> composersBx :: StateTBX  Identity 
>                          [(Name, Nation, Dates)] 
>                          (Set (Name, Nation, Dates)) 
>                          [(Name,Nation)]
> composersBx = StateTBX getl setl initl getr setr initr
>   where
>     getl = state (\l -> (Set.fromList l, l))
>     setl = (\m -> state (\l -> ((), f (Set.toList m) [] l))) 
>     initl = (\m -> return (Set.toList m))
>
>     f leftovers upd [] = (reverse upd) ++ sort leftovers
>     f leftovers upd ((nn,na,nd) : rs)  = f ns (ps ++ upd) rs
>       where (ps, ns) = selectNNDonKey nn leftovers 
> 
>     getr = state (\l -> (selectNNfromNND l, l))
>     setr = (\n -> state (\l -> ((), g n [] l))) 
>     initr = (\n -> return (mkNNDfromNN n))
>
>     g [] updated stateNotSeenYet = reverse updated
>     g (h@(nn,na) : todo) updated stateNotSeenYet =
>         g todo (newTriple : updated) ns
>       where (ps, ns) = selectNNDonKey nn stateNotSeenYet
>             newTriple = newTripleFromList h (\(_,_,d) -> d) ps

Now let's see how to use both the symmetric lens and the bx versions,
and demonstrate them behaving the same.

\begin{enumerate}

\item
Initialise both with no composers at all.

> (m1, c1) = putl composers ([], missing composers)
> s1 = runIdentity (initr composersBx [])

\item
Now suppose the owner of the left-hand view likes JS Bach.

> jsBach =  (  Name "J. S. Bach", Nation "German", 
>              Dates (Just (Date "1685", Date "1750")))
> onlyBach = Set.fromList([jsBach])

Putting this into the symmetric lens version:

> (n1sl, c2) = putr composers (onlyBach, c1)

and into the bx version (the underscore represents the result of the monadic
computation; we could use |evalState| if we didn't like it, but this is
just standard Haskell-monad-cruft, nothing to do with our formalism
specifically:

> (_,s2) = runState (do {setl composersBx onlyBach}) s1

\item
Let's check that what the owner of the right-hand view sees is the
same in both cases. n1 is that, for the symmetric lens (we got told,
whether we liked it or not). For the bx:

> (n1bx, s3) = runState (do {n <- getr composersBx; return n}) s2
> ok1 = (n1sl == n1bx)

\item
The RH view owner also likes John Tavener:

> johnTavener = (  Name "John Tavener", Nation "British")

and decides to append:

> bachTavener = n1sl ++ [johnTavener]

Putting this into the symmetric lens version:

> (m1sl, c3) = putl composers (bachTavener, c2)

and into the bx version:

> (_, s4) = runState (do {setr composersBx bachTavener}) s3

yields the same result for the LH view owner:

> (m1bx, s5) = runState (do {m <- getl composersBx; return m}) s4
> ok2 = (m1sl == m1bx) 

(Note that Haskell's |Set| equality compares the contents of |Set|s ignoring multiplicity and order.)

\item
The LH owner looks up Tavener's dates:

> datesJT = Dates (Just (Date "1944", Date "2013"))

and fixes their view:

> (yesJT, noJT) = Set.partition (\(nn,na,dd) -> nn == Name "John Tavener") m1sl
> fixedYesJT = Set.map (\(nn,na,dd) -> (nn,na,datesJT)) yesJT
> m2 = Set.union noJT fixedYesJT

and puts it back in the symmetric lens version:

> (n2sl, c4) = putr composers (m2, c3)

and in the bx version:

> (_,s6) = runState (do {setl composersBx m2}) s5

Checking result from the other side:

> (n2bx, s7) = runState (do {n <- getr composersBx; return n}) s6
> ok3 = (n2sl == n2bx) 

\item
Back to the RH view owner

> n3 =  ((  Name "Hendrik Andriessen", Nation "Dutch") : n2sl) 
>       ++ [(Name "J-B Lully", Nation "French")]
> (m3sl, c5) = putl composers (n3, c4)               
> (_,s8) = runState (do {setr composersBx n3}) s6

\end{enumerate}

To note:

\begin{itemize}

\item
we have shown alternating sets on the two sides, as that is natural
for symmetric lenses; for bx, any order of sets works equally well
(and there is no need to wonder about what complement to use).

\item
We've shown the fine-grained version to facilitate comparison, but
we can also combine steps:

> (n', s') = runIdentity (runr composersBx (do {
>              setl composersBx onlyBach; 
>              n <- getr composersBx; 
>              return n}) [])

etc. 

\end{itemize}

Auxiliary definitions; only the |Show| instance for Dates is noteworthy

> newtype Name = Name { unName :: String } 
>   deriving (Eq, Ord)
> instance Show Name where
>   show (Name n) = n
>
> newtype Nation = Nation { unNation :: String } 
>   deriving (Eq, Ord)
> instance Show Nation where
>   show (Nation n) = n
>
> newtype Date = Date { unDate :: String } 
>   deriving (Eq, Ord)
> instance Show Date where
>   show (Date d) = d
>
> newtype Dates = Dates { unDates :: Maybe (Date,Date) } 
>   deriving (Eq, Ord)
> instance Show Dates where
>   show (Dates d) = h d
>     where h Nothing = "????"
>           h (Just (dob,dod)) = show dob ++ "--" ++ show dod
>
> dates0 :: Dates 
> dates0  = Dates Nothing


